module Network.Lean.Lai (
      Application
    , Request(..)
    , Response(..)
    ) where

import Data.ByteString (ByteString)
import Data.Serialize.Ldsf.Types (Value(..), Text8)
import Network.Lean.Types
import Network.Lean.Lai.Internal

type Port = Int

type Application requestInternal responseInternal = Request requestInternal -> (Response responseInternal -> IO ResponseReceived) -> IO ResponseReceived

-- | The request object.
data Request internal = Request
    { requestHostname :: Text8 -- ^ The host to which this request should be routed to.
    , requestCategory :: InterfaceCategory -- ^ The 'InterfaceCategory' of the request.
    , requestVersion :: InterfaceVersion -- ^ The 'InterfaceVersion' of the request.
    , requestFunction :: InterfaceFunctionId -- ^ The 'InterfaceFunction' of the request.
    , requestArguments :: [Value] -- ^ The arguments as '[Value]'.
    , requestInternal :: Maybe internal -- ^ Variable to access implementation-specific state.
    } deriving (Show) -- only for debugging

-- | The response object.
data Response internal = Response
    { responseType :: ResponseType -- ^ The response type.
    , responsePayload :: [Value] -- ^ The payload as '[Value]'.
    , responseInternal :: Maybe internal -- ^ Variable to access implementation-specific state.
    } deriving (Show) -- only for debugging
