{-# LANGUAGE BangPatterns #-}
{-# LANGUAGE ScopedTypeVariables #-}
module Data.Serialize.Ldsf.Internal (
      fitBounded
    , splitInteger
    , combineInteger
    ) where

-- | Fit a value from a larger data type into a smaller data type.
-- | If the value does not fit, then the max value of the smaller data type
-- | is used.
fitBounded :: forall a b . (Bounded b, Ord a, Ord b, Integral a, Integral b) => a -> b
fitBounded value = bounded
    where
        bounded =
            if value > minValue
               then fromIntegral (min value maxValue) :: b
               else fromIntegral (max value minValue) :: b
        minValue = fromIntegral (minBound :: b) :: a
        maxValue = fromIntegral (maxBound :: b) :: a

-- | Take 'minBound' if negative, or 'maxBound' if positive from the
-- | 'Integer' towards 0.
{-# INLINE takeToZero #-}
takeToZero :: forall a . (Integral a, Bounded a) => Integer -> (Integer, a)
takeToZero largeNum = (leftover, fromInteger amount)
    where
        (comp, bound) =
            if largeNum < 0
                then (max, minBound :: a)
                else (min, maxBound :: a)
        amount = comp largeNum (fromIntegral bound)
        leftover = largeNum - amount

-- | Split an 'Integer' into a list of smaller integers.
splitInteger :: forall a . (Integral a, Bounded a) => Integer -> [a]
splitInteger = si []
    where
        {-# INLINE si #-}
        si :: [a] -> Integer -> [a]
        si !acc 0 = acc
        si !acc !l = si (x : acc) leftover
            where
                (leftover, x) = takeToZero l

-- | Sum a list of 'Integral' into an 'Integer'.
combineInteger :: Integral a => [a] -> Integer
combineInteger = sum . map fromIntegral
