module Network.Lean.Client (
      HostName
    , ServiceName
    , Header
    , ClientRequest
    , ClientResponse
    , Request(..)
    , Response(..)
    , Client( clientHostname
            , clientPort
            )
    , createClient
    , destroyClient
    , withClient
    , request
    ) where

import Control.Exception (bracket)
import Data.Serialize.Get (Get)
import Data.Serialize.Ldsf (Header)
import qualified Data.Serialize.Ldsf as Ldsf
import qualified Data.Serialize.Ldsf.Types as Ldsf
import Data.Serialize.Ldsf.Lean (composeRequest)
import qualified Data.Serialize.Ldsf.Lean.InterfaceFunction as IF
import Network.Lean.Client.Internal
import Network.Lean.Lai
import Network.Lean.Types
import Network.Simple.TCP (HostName, ServiceName, Socket, SockAddr
                          , connectSock, closeSock, send)

type ClientRequest = Request ClientRequestState
type ClientResponse = Response ClientResponseState

-- | Simple client for the lean protocol.
data Client = Client
    { clientHostname :: String
    , clientPort :: String
    , clientSocket :: Socket
    , clientSocketAddr :: SockAddr
    }

createClient :: HostName -> ServiceName -> IO Client
createClient hostname port = do
    (socket, socketAddr) <- connectSock hostname port
    pure Client
        { clientHostname = hostname
        , clientPort = port
        , clientSocket = socket
        , clientSocketAddr = socketAddr
        }

destroyClient :: Client -> IO ()
destroyClient client = closeSock $ clientSocket client

withClient :: HostName -> ServiceName ->
              (Client -> IO (Maybe (Header, ClientResponse))) ->
              IO (Maybe (Header, ClientResponse))
withClient hostname port thing =
    bracket
        (createClient hostname port)
        (destroyClient)
        thing

-- TODO: FIXME: error handling ????
request :: Client -> ClientRequest -> IF.GetResult -> IO (Maybe (Header, ClientResponse))
request client request getResult = do
    send socket request'
    handleResponse getResult socket
    where
        socket = clientSocket client
        request' = composeRequest request

-- | TODO: error handling mechanisms?
--requestSingle :: HostName -> ServiceName -> Request ->
--           IF.GetResult ->
--           IO (Maybe (Header, Response))
--requestSingle host port req getResult = do
--    connect host port $ \(socket, remote) -> do
--        let reqdata = composeRequest req
--        send socket reqdata
--        handleResponse getResult socket
