{-# LANGUAGE OverloadedStrings #-}
module Main where

import qualified Data.ByteString as B
import qualified Data.ByteString.Lazy as BL
import Data.Text.Lazy (Text)
import qualified Data.Text.Lazy as T
import qualified Data.Text.Lazy.Encoding as T
import qualified Data.Text.Lazy.IO as T
import Network.HTTP.Types
import Network.Wai
import Network.Wai.Handler.Warp (run)

-- !!README!!

-- This is just a simple, hard-coded server to emulate what
-- it would be like to browse the lean web.

-- This is _NOT_ for production use!
-- This is _NOT_ what the resulting server would be like!
-- This is _STRICTLY_ for _TESTING_ purposes only!
-- This only exists because we don't have a proper server implementation yet!

app :: Application
app request respond = respond $ case rawPathInfo request of
    "/"             -> responseIndex
    "/another-page" -> responseAnotherPage
    "/evil-page"    -> responseEvilPage
    _               -> response404

-- Yes, to save time we really just concatenate strings instead of
-- using type-safe records which then use Aeson to convert them to their
-- JSON string representation.
makeFakeJson :: Text -> Text
makeFakeJson txt = T.concat [ "{\"content\":\"", txt, "\"}" ]

makeFakeIndex = makeFakeJson $ T.concat
    [ "Hi, this is crowbars _fake_ web page!\n\n"
    , "It's fake because we don't actually have a real server yet, we are just emulating what it would be like to browse the lean web.\n"
    , "It's quicker to test ideas this way :).\n\n"
    , "Here are some links:\n"
    , "lean://localhost:8080/\n"
    , "lean://localhost:8080/another-page\n"
    , "lean://localhost:8080/evil-page SCARY!!!"
    ]

makeFakeAnotherPage = makeFakeJson $ T.concat
    [ "This is another page... wooooo.....\n"
    , "You can go back now using the following link:\n\n"
    , "lean://localhost:8080/"
    ]

makeFakeEvilPage = makeFakeJson $ T.concat
    [ "This is an evil page which contains non-printable ASCII characters.\n"
    , "Go back: lean://localhost:8080/\n\n"
    , "TODO: actually insert all 256 values of a byte here."
    ]

makeFake404 = makeFakeJson "404 - we don't actually have a mechanism for this yet."

makeFakeResponse :: Text -> Response
makeFakeResponse fakeTextContent = responseLBS
    status200
    [( "Content-Type", "application/json" )]
    (T.encodeUtf8 fakeTextContent)

responseIndex :: Response
responseIndex = makeFakeResponse makeFakeIndex

responseAnotherPage :: Response
responseAnotherPage = makeFakeResponse makeFakeAnotherPage

responseEvilPage :: Response
responseEvilPage = makeFakeResponse makeFakeEvilPage

response404 :: Response
response404 = makeFakeResponse makeFake404
    
main :: IO ()
main = do
    let port = 8080 :: Int
    T.putStrLn $ T.concat [ "Running server on port: ", T.pack $ show port ]
    run port app
